@@ -11,10 +11,15 @@ if not modules then modules = { } end modules ['mtx-server-ctx-fonttest'] = {
 dofile(resolvers.findfile("trac-lmx.lua","tex"))
 dofile(resolvers.findfile("font-ini.lua","tex"))
 dofile(resolvers.findfile("font-con.lua","tex"))
-dofile(resolvers.findfile("font-oti.lua","tex"))
-dofile(resolvers.findfile("font-otf.lua","tex"))
-dofile(resolvers.findfile("font-otp.lua","tex"))
-dofile(resolvers.findfile("font-ott.lua","tex"))
+-- dofile(resolvers.findfile("font-oti.lua","tex"))
+-- dofile(resolvers.findfile("font-otf.lua","tex"))
+-- dofile(resolvers.findfile("font-otp.lua","tex"))
+dofile(resolvers.findfile("font-otr.lua","tex"))
+dofile(resolvers.findfile("font-cff.lua","tex"))
+-- dofile(resolvers.findfile("font-ttf.lua","tex"))
+-- dofile(resolvers.findfile("font-ott.lua","tex"))
+dofile(resolvers.findfile("font-dsp.lua","tex"))
+dofile(resolvers.findfile("font-onr.lua","tex"))
 dofile(resolvers.findfile("font-syn.lua","tex"))
 dofile(resolvers.findfile("font-mis.lua","tex"))
 
